import sys
import os
import glob
from os.path import basename
from os.path import dirname
import posixpath
from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.locale import _


from sphinx.util import logging
import logging


from xml.etree import ElementTree

from sphinx.builders import Builder



class Pdf(nodes.Inline, nodes.Element):
	"""PDF Node"""



def visit_pdf_node(self, node):
	
	#print("Visit pdf: ",self.builder,"\n")
	
	finalPath = posixpath.join(self.builder.dlpath,node.fileName)
	self.builder.env.dlfiles[node.file] = [node.file,node.fileName]

	containerClasses = ""
	if node.sticky is True:
		containerClasses = containerClasses +" ui sticky"

	#self.builder.images[node.file]= "lecture"
	classes = "pdfjs"
	

	self.body.append('<div id="pdfjs-container" class="%s">' % (containerClasses))
	self.body.append('<canvas class="%s" data-file="%s"></canvas>' %(classes,finalPath))
	self.body.append('<div class="pdfjs-controls">')
	self.body.append('<i class="ui left arrow icon" onclick="pdfjs.previousPage(this);"></i>')
	self.body.append('<i class="ui right arrow icon" onclick="pdfjs.nextPage(this);"></i>')
	self.body.append('</div>')

def depart_pdf_node(self, node):
	self.body.append('</div>')


#self.body.append('</canvas>')

class OdfiPdfPageDirective(Directive):

	optional_arguments = 1

	def run(self):

		print("PDF Page: ",self.arguments[0])



class OdfiPdfDirective(Directive):
	has_content = True
	optional_arguments = 10
	def run(self):

		pdfNode = Pdf()

		## Get File, relative to current source
		###############

		fileName = self.arguments[0]
		env = self.state.document.settings.env
		source = self.state_machine.input_lines.source(self.lineno - self.state_machine.input_offset - 1)
		sourceDirectory = dirname(source)
		pdfFile  = os.path.normpath(sourceDirectory+"/"+fileName)

		## Normalize name because of upper case issues on windows
		pdfFile = "%s[%s]" % (pdfFile[:-1], pdfFile[-1])
		pdfFile = glob.glob(pdfFile)[0]

		if os.path.exists(pdfFile):
			print("PDF File is at: ",pdfFile)
		else:
			return [nodes.error(None, nodes.paragraph(text = "Unable to Load PDF File at %s:%d: file %s does not exist" % (basename(source), self.lineno,pdfFile))) ]


		pdfNode.fileName  = fileName
		pdfNode.file = pdfFile

		## Set content
		pdfNode.content = self.content

		## Sticky
		if ":sticky:" in self.arguments:
			pdfNode.sticky = True
		else:
			pdfNode.sticky = False



		return [pdfNode]




def setup(app):

	"""html=(visit_OdfiSVG_node, depart_OdfiSVG_node)"""
	app.add_node(Pdf, html=(visit_pdf_node, depart_pdf_node))
	app.add_directive('odfi.pdf',OdfiPdfDirective)
	app.add_role('odfi.pdfPage',OdfiPdfPageDirective)


	return {'version': '1.0.0'}


import sys
import os
from os.path import basename
from os.path import dirname

from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.locale import _


from sphinx.util import logging
import logging


from xml.etree import ElementTree

class Code(nodes.Inline, nodes.Element):
	"""SVG Node"""



def visit_code_node(self, node):
	if hasattr(node,"language"):
		lang=('class="%s"' % node.language)
	else:
		lang=""
	self.body.append('<pre>')
	self.body.append('<code %s>' % lang)
	self.body.append("\n".join(node.content))

def depart_code_node(self, node):
	self.body.append('</code>')
	self.body.append('</pre>')



class OdfiCodeDirective(Directive):
	has_content = True
	optional_arguments = 1
	def run(self):

		codeNode = Code()

		## Get language arg
		print(self.arguments)
		if len(self.arguments) == 1:
			codeNode.language = self.arguments[0]

		## Set content
		codeNode.content = self.content
		print(self.content)

		return [codeNode]

def setup(app):

	"""html=(visit_OdfiSVG_node, depart_OdfiSVG_node)"""
	app.add_node(Code, html=(visit_code_node, depart_code_node))
	app.add_directive('odfi.code',OdfiCodeDirective)
	return {'version': '1.0.0'}



import sys
import os
import glob
from os.path import basename
from os.path import dirname

from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.locale import _


from sphinx.util import logging
import logging
import re

from zipfile import ZipFile

from xml.etree import ElementTree



class StepsRecorder(nodes.Inline, nodes.Element):
	"""StepsRecorder Node"""



def visit_StepsRecorder_node(self, node):

	## Extract XML
	mainHTML = node.resultParts["main.htm"]
	#print("Search HTML: "+mainHTML)
	reportXML = re.search("<Report>.*</Report>",mainHTML)
	if reportXML is None:
		print("Report not found inside: ",mainHTML.encode(sys.stdout.encoding, errors='replace'))

	#print("Found Report: "+reportXML.group(0))

	report = ElementTree.fromstring(reportXML.group(0))

	## Create For each step a Div
	self.body.append('<div class="steps">')

	## File Name

	## Action
	eachAction = report.findall("./UserActionData/RecordSession/EachAction")

	## TOC?
	if node.stepsToc is True:
		counter = 0
		self.body.append('<ol class="steps-toc">')
		for action in eachAction: 
			number =  action.get("ActionNumber")
			counter += 1
			if action.get("ActionNumber") in node.resultParts:
				self.body.append('<li><a href="#%s-step%d">%s</a></li>' % (node.stepsName,counter,node.resultParts[action.get("ActionNumber")]))
		self.body.append('</ol>')

	#self.body.append('<div class="steps-header">Steps below:</div>')
	counter = 0 
	for action in eachAction: 
		counter += 1
		number =  action.get("ActionNumber")
		print("Action: "+number)
		self.body.append('<div class="step" id="%s-step%d">' %(node.stepsName,counter))


		if action.get("ActionNumber") in node.resultParts:
			self.body.append('<div class="caption"> %s </div>' %  node.resultParts[action.get("ActionNumber")])

			## Add Picture
			self.body.append('<div class="picture">')
			self.body.append('<img src="data:image/jpeg;base64,%s"/>' % node.resultParts[action.findtext("ScreenshotFileName")])
			self.body.append('</div>')

		# EOF Step
		self.body.append('</div>')
		
	
	self.body.append('</div>')

	pass
	

def depart_StepsRecorder_node(self, node):
	pass

class OdfiStepsRecorderDirective(Directive):

  
	has_content = True
	required_arguments = 1 
	optional_arguments = 64
           

	def run(self):

		print('Running ODFI Steps Recorder Directive')
		print( self.arguments)
		print( self.options)
		print( self.content)
		
		env = self.state.document.settings.env
		source = self.state_machine.input_lines.source(self.lineno - self.state_machine.input_offset - 1)

		## Get File, relative to current source
		fileName = self.arguments[0]
		sourceDirectory = dirname(source)
		stepsFile  = os.path.normpath(sourceDirectory+"/"+fileName)

		## Normalize name because of upper case issues on windows
		stepsFile = "%s[%s]" % (stepsFile[:-1], stepsFile[-1])
		stepsFile = glob.glob(stepsFile)[0]

		if os.path.exists(stepsFile):
			print("Steps File is at: ",stepsFile)
		else:
			return [nodes.error(None, nodes.paragraph(text = "Unable to Load Steps File at %s:%d: file %s does not exist" % (basename(source), self.lineno,stepsFile))) ]


		## Read file and split parts
		#################
		stepsNode = StepsRecorder()
		stepsNode.stepsName = fileName

		stepsBlock = nodes.container();

		##  Steps File can be a Zip File
		if stepsFile.endswith(".zip"):
			with ZipFile(stepsFile) as stepsZipFile:
				with stepsZipFile.open(stepsZipFile.namelist()[0]) as stepsFileEntry:
					stepsContent = stepsFileEntry.read().decode("cp1252").replace('\r','')
					#print("Steps content from zip: ",stepsContent.encode(sys.stdout.encoding, errors='replace'))

		else:
			with open(stepsFile) as f: stepsContent = f.read()
		## get boundary
		match = re.search('boundary="([\w_=\+\.\-]+)"',stepsContent)
		boundary= "--"+match.group(1)
		print("Steps Boundary: "+boundary)

		## Split
		parts = stepsContent.split(boundary)
		print("Parts: %s" % len(parts))

		## Store Each Part with name
		resultParts = {}
		for partContentIndex in range(1, len(parts)):

			## Get Part content line by line, to extract parameters
			## iF an empty line is encountered, it is the beginning of the content
			partContent = parts[partContentIndex].strip()
			partFileContent = ""
			fileName = ""
			contentStart = False
			lines = partContent.split("\n")
			for lineIndex in range(0, len(lines)):
				
				line = lines[lineIndex]
				
				if contentStart is True: 
					partFileContent+=line
				elif line.startswith("Content-Location:"):
					fileName =  line.strip("Content-Location:").strip()
					print("Part File Name: "+fileName)
				elif line == "":
					print("Found Content Start")
					contentStart = True 

			# Save part file content
			resultParts[fileName] = partFileContent

			## Make admonition

		## Save result Parts to node 
		stepsNode.resultParts = resultParts

		## Read Directive Content and save Step Description
		for contentLine in self.content: 
			match = re.search("([\d]+)\s+(.*)",contentLine)
			number = match.group(1)
			desc = match.group(2)
			print("Part desc: ",desc)
			resultParts[number] = desc

		## Add some more options
		if "toc" in self.arguments:
			stepsNode.stepsToc = True
		else:
			stepsNode.stepsToc = False

		if "slideshow" in self.arguments:
			stepsNode.slideshow = True
		else:
			stepsNode.slideshow = False



		return [stepsNode]

def setup(app):

	"""html=(visit_OdfiSVG_node, depart_OdfiSVG_node)"""
	app.add_node(StepsRecorder, html=(visit_StepsRecorder_node, depart_StepsRecorder_node))
	app.add_directive('odfi.stepsrecorder',OdfiStepsRecorderDirective)
	return {'version': '1.0.0'}



import sys
import os
from os.path import basename
from os.path import dirname

from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.locale import _


from sphinx.util import logging
import logging


from xml.etree import ElementTree


class svg(nodes.Inline, nodes.Element):
	"""SVG Node"""



def visit_svg_node(self, node):

	if hasattr(node,'groupId'):

		## Find Group content in source
		ns = {'svg': 'http://www.w3.org/2000/svg', 'inkscape' : 'http://www.inkscape.org/namespaces/inkscape'}
		svgDom = ElementTree.parse(node.source)
		svgRoot = svgDom.getroot()

		## Look for element by ID
		foundElement = svgRoot.find(".//*[@id='%s']" %(node.groupId),ns)
		print("Found id %s -> %s " %(node.groupId,foundElement))

		## Handle: Sindle element or Groups
		#########
		if foundElement is None : 
			return

		print("Found tag2:  %s " %(foundElement.tag))

		## Find defs in source
		foundDefs = svgRoot.find(".//svg:defs",ns)

		if foundElement.tag == "{http://www.w3.org/2000/svg}g" :

			foundGroup = foundElement
			#foundGroup = svgRoot.find(".//svg:g[@id='%s']" %(node.groupId),ns)
			foundGroup.set("transform","")

			

			## Find top most Y (lowest y) to set transformation to get the group back on top
			allwithy = foundGroup.findall(".//svg:rect[@y]",ns) + foundGroup.findall(".//svg:image[@y]",ns)
			allwithx = allwithy

			## First calculate translation in X and Y to substract from all X/Y coordinates
			## Sort X list by most Right first
			## Sort Y list by most down first
			## Take last element as most left and most down
			mostRightFirst = sorted(allwithx,key=lambda n: float(n.get("x")),reverse=True)
			mostDownFirst  = sorted(allwithy,key=lambda n: float(n.get("y")),reverse=True)
			
			minX = float(mostRightFirst[-1].get("x"))
			minY = float(mostDownFirst[-1].get("y"))

			## From here translate all X/Y coordnates because we place back the group to 0/0
			## This way sizes are correct
			allXAndWidth = filter(lambda elt: elt.get("x")!=None and elt.get("width")!=None,allwithx)
			allMostRightCoordinates = sorted(map(lambda elt: float(elt.get("x")) - minX + float(elt.get("width")), allXAndWidth),reverse=True)
			
			allYAndHeight = filter(lambda elt: elt.get("y")!=None and elt.get("height")!=None,allwithy)
			allMostDownCoordinates = sorted(map(lambda elt: float(elt.get("y")) - minY + float(elt.get("height")), allYAndHeight),reverse=True)

			

			maxX = allMostRightCoordinates[0]
			maxY = allMostDownCoordinates[0]

		else:



			minX = float(foundElement.get("x"))
			minY = float(foundElement.get("y"))

			maxX = float(foundElement.get("x")) - minX + float(foundElement.get("width"))
			maxY = float(foundElement.get("y")) - minY + float(foundElement.get("height"))


		
		## Modify Properties of SVG now
		################################



		## Create new SVG 
		newSvg = ElementTree.fromstring('<?xml version="1.0" encoding="UTF-8" standalone="no"?><svg version="1.1"  xmlns="http://www.w3.org/2000/svg"></svg>')
		

		

		#ally = map(lambda n: float(n.get("y")),allwidthy)
		#allx = map(lambda n: float(n.get("x")),allwidthx)
		#allwidth = map(lambda n: float(n.get("width")),allwidthx)
		#allheight = map(lambda n: float(n.get("height")),allwidthx)
		#miny = min(map(lambda n: float(n.get("y")),allwidthy))
		#minx = min(map(lambda n: float(n.get("x")),allwidthx))

		print("Found Min y 2: ",minY)
		print("Found Min x 2: ",minX)
		print("Found Max y 2: ",maxY)
		print("Found Max x 2: ",maxX)

		## Translate Group
		#translateX = 0 if minX < 0 else minX
		#translateY = 0 if minY < 0 else minY
		translateX = minX
		translateY = minY
		foundElement.set("transform","translate(%s,%s)" % (-translateX,-translateY))
	

		## View Box is 0 0 maxX maxY
		## Min and max are already translated
		viewBox = "0 0 %s %s" % (maxX,maxY) 
		newSvg.set("viewBox",viewBox)


		##  add group to it
		newSvg.insert(0,foundElement)

		## Add Defs
		newSvg.insert(0,foundDefs)

		
		ElementTree.register_namespace("","http://www.w3.org/2000/svg")
		ElementTree.register_namespace("xlink","http://www.w3.org/1999/xlink")
		ElementTree.register_namespace("inkscape","http://www.inkscape.org/namespaces/inkscape")
		
		str = ElementTree.tostring(newSvg, encoding='utf8', method='xml').decode("utf-8") 
		#print("Found svg: ",str)

		## DIV Container and Style
		####################
		targetStyle= node.style if hasattr(node,'style') else ""

		if hasattr(node,'style_center'):
			targetStyle = targetStyle + ";margin-left:auto;margin-right:auto"

		if hasattr(node,'style_width'):
			targetStyle = targetStyle + (";width:%s" % node.style_width)

		## Set Container Div width provided style if defined
		self.body.append('<div class="odfi svg" style="%s">' % (targetStyle))
		self.body.append(str)
		self.body.append('</div>')

	else:
		with open(node.source) as f: s = f.read()

		targetStyle= node.style if hasattr(node,'style') else ""
		if hasattr(node,'style_center'):
			targetStyle = targetStyle + ";margin-left:auto;margin-right:auto"

		if hasattr(node,'style_width'):
			targetStyle = targetStyle + (";width:%s" % node.style_width)

		self.body.append('<div class="odfi svg" style="%s">' % (targetStyle))
		self.body.append(s)
		self.body.append('</div>')



def depart_svg_node(self, node):
	pass

class OdfiSVGDirective(Directive):

  
	has_content = True
	required_arguments = 2 
	optional_arguments = 64
           

	def run(self):

		print('Running ODFI SVG Directive')
		print( self.arguments)
		print( self.options)
		
		env = self.state.document.settings.env
		source = self.state_machine.input_lines.source(self.lineno - self.state_machine.input_offset - 1)

		
		#logger = logging.getLogger(__name__)
		#logger.info('Running ODFI SVG Directive')

		## Get File
		fileName = self.arguments[0]
		
		#if not "file" in self.arguments: 
		#	return [nodes.error(None, nodes.paragraph(text = "Unable to Load SVG at %s:%d: file argument not defined" % (basename(source), self.lineno))) ]
		
		## Get File name relative to local source
		#fileName = self.arguments[(self.arguments.index('file') +1)]
		sourceDirectory = dirname(source)
		sourceSVG  = os.path.normpath(sourceDirectory+"/"+fileName)

		if os.path.exists(sourceSVG):
			print("SVG File is at: ",sourceSVG)
		else:
			return [nodes.error(None, nodes.paragraph(text = "Unable to Load SVG at %s:%d: file %s does not exist" % (basename(source), self.lineno,sourceSVG))) ]

		
		## Create SVG Node
		targetid = "svg-%d" % env.new_serialno('svg')
		svgNode = svg()
		svgNode.source = sourceSVG
	
		## Set Target Group Id if necessary
		if "gid" in self.arguments: 
			print('Found Elementid')
			svgNode.groupId = self.arguments[(self.arguments.index('gid') +1)]

		

		## Style Arguments
		######################
		## Set Style if necessary
		if "style" in self.arguments: 
			svgNode.style = self.arguments[(self.arguments.index('style') +1)].replace('"',"").replace("'","")

		if "align" in self.arguments and self.arguments[(self.arguments.index('align') +1)] == "center":
			svgNode.style_center = True

		if ":width:" in self.arguments:
			svgNode.style_width = self.arguments[(self.arguments.index(':width:') +1)]


		## Animation Arguments
		######################

		return [svgNode]


def setup(app):

	"""html=(visit_OdfiSVG_node, depart_OdfiSVG_node)"""
	app.add_node(svg, html=(visit_svg_node, depart_svg_node))
	app.add_directive('odfi.svg',OdfiSVGDirective)
	return {'version': '1.0.0'}






import sys
import os
import glob
from os.path import basename
from os.path import dirname
import posixpath
from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.locale import _
from sphinx.domains import _

from sphinx.util import logging
import logging


from xml.etree import ElementTree

from sphinx.builders import Builder
from sphinx.domains  import Domain 

class TraitDirective(Directive):
	""" Directive to set trait"""


class ScalaDomain(Domain):
	"""Scala Domain"""
	name = "scala"
	label = "Scala Domain"
	directives = {}
	directives['trait'] = TraitDirective




def setup(app):

	"""html=(visit_OdfiSVG_node, depart_OdfiSVG_node)"""
	app.add_config_value("maven_projects",[],'env')
	app.add_domain(ScalaDomain)
	#app.add_directive_to_domain('scala','trait',TraitDirective)


""" Sphinx ODFi Theme
"""
import os

__version__ = '1.0.1'
__version_full__ = __version__


def get_html_theme_path():
    """Return list of HTML theme paths."""
    cur_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    return cur_dir


import sys
import os
import glob
from os.path import basename
from os.path import dirname
import posixpath
from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.locale import _
from sphinx.domains import _

from sphinx.util import logging
import logging


from xml.etree import ElementTree

from sphinx.builders import Builder
from sphinx.domains  import Domain 



class Question(nodes.Admonition, nodes.TextElement):
	"""Node for questions.
	"""

class QuestionDirective(Directive):

    # this enables content in the directive
	has_content = True

	def run(self):

		env = self.state.document.settings.env

		targetid = "question-%d" % env.new_serialno('question')
		targetnode = nodes.target('', '', ids=[targetid])
 

		ad = Question('\n'.join(self.content))
		ad += nodes.title(_('Question'), _('Question'))
		#ad += nodes.class(_('Question'), _('Question'))
		self.state.nested_parse(self.content, self.content_offset, ad)
		ad.attributes["classes"] =  ["question"]
		#ad.options["class"] = "question"

		#ad = Question()
		#ad.name = self.name 
		#ad.content_offset = self.content_offset 
		#ad.options = self.options 
		#ad.content = self.content 
		#ad.block_text = self.block_text
		#ad.state = self.state
		#ad.state_machine = self.state_machine

		#ad = make_admonition(Question, self.name, [_('Question')], self.options,
		#					self.content, self.lineno, self.content_offset,
		#					self.block_text, self.state, self.state_machine)

		return [targetnode,ad]


def visit_question_node(self, node):
	self.visit_admonition(node)

def depart_question_node(self, node):
	self.depart_admonition(node)


def setup(app):

	app.add_node(Question, html=(visit_question_node, depart_question_node))
	app.add_directive('odfi.question',QuestionDirective)
	return {'version': '1.0.0'}